"use strict";
// -*- mode: typescript; indent-tabs-mode: nil; js-basic-offset: 4 -*-
//
// This file is part of ThingTalk
//
// Copyright 2018-2021 The Board of Trustees of the Leland Stanford Junior University
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
// Author: Giovanni Campagna <gcampagn@cs.stanford.edu>
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const visitor_1 = __importDefault(require("./visitor"));
const program_1 = require("./program");
const dataset_1 = require("./dataset");
const statement_1 = require("./statement");
function declarationLikeToProgram(self) {
    const nametoslot = {};
    let i = 0;
    for (const name in self.args)
        nametoslot[name] = i++;
    let declarations, statements;
    if (self instanceof dataset_1.Example) {
        declarations = [];
        statements = [new statement_1.ExpressionStatement(null, self.value.clone())];
    }
    else {
        declarations = self.declarations.map((d) => d.clone());
        statements = self.statements.map((s) => {
            if (s instanceof statement_1.ReturnStatement)
                return new statement_1.ExpressionStatement(s.location, s.expression.clone());
            else
                return s.clone();
        });
    }
    const program = new program_1.Program(null, [], declarations, statements);
    program.visit(new class extends visitor_1.default {
        visitVarRefValue(value) {
            if (value.name in nametoslot)
                value.name = '__const_SLOT_' + nametoslot[value.name];
            return true;
        }
    });
    return program;
}
exports.default = declarationLikeToProgram;
//# sourceMappingURL=declaration_like_to_program.js.map