export interface Gettext {
    locale: string;
    gettext: (x: string) => string;
    dgettext: (d: string, x: string) => string;
    ngettext: (x1: string, xn: string, n: number) => string;
    dngettext: (d: string, x1: string, xn: string, n: number) => string;
}
/**
 * The node-gettext library.
 * @external Gettext
 * @see {@link https://www.npmjs.com/package/node-gettext}
 */
/**
 * Internationalization support.
 *
 * @alias I18n
 * @namespace
 */
/**
 * Initialize internationalization support.
 *
 * This function should be called before any other API in the ThingTalk library.
 * It might be called multiple times to initialize multiple locales (e.g. in
 * a multi-user application).
 *
 * The passed-in gettext instance must have the `thingtalk` domain loaded with
 * the .mo files in this package.
 *
 * @param {string} locale - the locale to initialize, as a BCP 47 tag
 * @param {external:Gettext} gettext - the initialized gettext instance for this locale
 */
export declare function init(locale: string, gettext: Gettext): void;
/**
 * Retrieve translations for a given locale.
 *
 * If the locale is unsupported, this method will return a valid Gettext-like object
 * that returns no translation.
 *
 * @param {string} locale - the locale to retrieve
 * @return {external:Gettext} the gettext instance in the given locale, with loaded translations
 */
export declare function get(locale: string): Gettext;
