import type * as Ast from './ast';
import { TokenStream } from './new-syntax/tokenstream';
export declare type TypeMap = ({
    [key: string]: Type;
});
export declare type TypeScope = ({
    [key: string]: Type | string;
});
export declare type EntitySubTypeMap = Record<string, string[]>;
interface Hashable<T> {
    hash(): number;
    equals(other: T): boolean;
}
/**
 * The base class of all ThingTalk types.
 */
export default abstract class Type implements Hashable<Type> {
    static Any: Type;
    isAny: boolean;
    static Boolean: Type;
    isBoolean: boolean;
    static String: Type;
    isString: boolean;
    static Number: Type;
    isNumber: boolean;
    static Currency: Type;
    isCurrency: boolean;
    static Entity: typeof EntityType;
    isEntity: boolean;
    static Measure: typeof MeasureType;
    isMeasure: boolean;
    static Enum: typeof EnumType;
    isEnum: boolean;
    static Array: typeof ArrayType;
    isArray: boolean;
    static Time: Type;
    isTime: boolean;
    static Date: Type;
    isDate: boolean;
    static RecurrentTimeSpecification: Type;
    isRecurrentTimeSpecification: boolean;
    static Location: Type;
    isLocation: boolean;
    static ArgMap: Type;
    isArgMap: boolean;
    static Object: Type;
    isObject: boolean;
    static Compound: typeof CompoundType;
    isCompound: boolean;
    static Unknown: typeof UnknownType;
    isUnknown: boolean;
    static fromString(str: Type | string): Type;
    prettyprint(): string;
    isNumeric(): boolean;
    isComparable(): boolean;
    abstract toSource(): TokenStream;
    abstract hash(): number;
    abstract equals(other: Type): boolean;
    static resolve(type: Type | string, typeScope: TypeScope): Type;
    static isAssignable(type: Type, assignableTo: Type | string, typeScope?: TypeScope, entitySubTypeMap?: EntitySubTypeMap): boolean;
}
export declare class EntityType extends Type {
    type: string;
    constructor(type: string);
    toString(): string;
    toSource(): TokenStream;
    hash(): number;
    equals(other: Type): boolean;
}
export declare class MeasureType extends Type {
    unit: string;
    constructor(unit: string);
    toString(): string;
    toSource(): TokenStream;
    hash(): number;
    equals(other: Type): boolean;
}
export declare class EnumType extends Type {
    entries: string[] | null;
    constructor(entries: string[] | null);
    toString(): string;
    toSource(): TokenStream;
    hash(): number;
    equals(other: Type): boolean;
}
export declare class ArrayType extends Type {
    elem: Type | string;
    constructor(elem: Type | string);
    toString(): string;
    toSource(): TokenStream;
    hash(): number;
    equals(other: Type): boolean;
}
declare type FieldMap = {
    [key: string]: Ast.ArgumentDef;
};
export declare class CompoundType extends Type {
    name: string | null;
    fields: FieldMap;
    private _hash;
    constructor(name: string | null, fields: FieldMap);
    toString(): string;
    toSource(): TokenStream;
    hash(): number;
    equals(other: Type): boolean;
}
export declare class UnknownType extends Type {
    name: string;
    constructor(name: string);
    toString(): string;
    toSource(): TokenStream;
    hash(): number;
    equals(other: Type): boolean;
}
export {};
